/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <generic.h>
#include <config.h>

/* mixxing routines
   these mix a short* 16bit input pcm buffer on top of a 32bit int* buffer
   the input buffer is interleaved stereo 16bit (samplerate is function dependant)
   the mixxing buffer is interleaved stereo 32bit 44khz
   argument num is number of bytes into the buffer to be mixed
   volume multiplier is also applied here */

int mixxx_mono_16(int *dest, short *chan, unsigned int num, float volume) {
  int c, off, tnum = (num/36);
  int delta = (num << 16) / tnum;
  float l;
  for(c=0,off=0;c<tnum;c++,off += delta) {
    l = chan[off >> 16]*volume;
    dest[c<<1] = (int) (dest[c<<1]+l);
    dest[(c<<1)+1] = (int) (dest[(c<<1)+1]+l);
  }  
  return(c);
}

int mixxx_stereo_16(int *dest, short *chan, unsigned int num, float volume) {
  int c, off,tnum = (num/36);
  int delta = (num << 16) / tnum;
  for(c=0,off=0;c<tnum;c++, off += delta)
    dest[c] = (int) (dest[c] + (chan[off >> 16]*volume));

  return(c); 
}

int mixxx_mono_22(int *dest, short *chan, unsigned int num, float volume) {
  unsigned int c, off, tnum = (num<<1);
  unsigned int delta = (num << 16) / tnum;
  float l;
  for(c=0,off=0;c<tnum;c++,off += delta) {
    l = chan[off >> 16]*volume;
    dest[c<<1] = (int) (dest[c<<1]+l);
    dest[(c<<1)+1] = (int) (dest[(c<<1)+1]+l);
  }  
  return(c);
}

int mixxx_stereo_22(int *dest, short *chan, unsigned int num, float volume) {
  unsigned int c, off,tnum = (num<<1);
  unsigned int delta = (num << 16) / tnum;
  for(c=0,off=0;c<tnum;c++, off += delta)
    dest[c] = (int) (dest[c] + (chan[off >> 16]*volume));

  return(c);
  
}
/*
int mixxx_mono_32(int *dest, short *chan, int num, float volume) {
  int c, off, tnum = (num/8); // (int)fabs(num*0.72562358277); //(int)fabs(num*0.97);
  int delta = (num << 16) / tnum;
  float l;
  for(c=0,off=0;c<tnum;c++,off += delta) {
    l = chan[off >> 16]*volume;
    dest[c<<1] = (int) (dest[c<<1]+l);
    dest[(c<<1)+1] = (int) (dest[(c<<1)+1]+l);
  }  
  return(c);
}

int mixxx_stereo_32(int *dest, short *chan, int num, float volume) {
  int c, off,tnum = (int)fabs(num/1.3);
  int delta = (num << 16) / tnum;
  for(c=0,off=0;c<tnum;c++, off += delta)
    dest[c] = (int) (dest[c] + (chan[off >> 16]*volume));

  return(c); 
}
*/

int mixxx_mono_44(int *dest, short *chan, unsigned int num, float volume) {
  unsigned int c,cc;
  float l;
  
  for(c=0;c<num;c++) {
    cc = c<<1;
    l = chan[c]*volume;
    
    dest[cc] = (int) (dest[cc]+l);
    dest[cc+1] = (int) (dest[cc+1]+l);
  }

  return(c);

}

int mixxx_stereo_44(int *dest, short *chan, unsigned int num, float volume) {
  unsigned int c;

  for(c=0;c<num;c++)
    dest[c] = (int) (dest[c]+(chan[c]*volume));

  return(c);
}

int mixxx_stereo_44_novol(int *dest, short *chan, int num) {
  int c;
  
  for(c=0;c<num;c++)
    dest[c] = (int) (dest[c] + chan[c]);
  
  return(c);
}

/* the following is borrowed from xmms pitch plugin by jammet@lionking.org
   thanks to newmark for help in making it fit here */

#define DO_PITCH(type, length, newlength, buffer, newbuffer) \
{ \
	int delta = (length << 16) / newlength, i, off; \
	type *ptr = (type *)buffer, *newptr = (type *)newbuffer; \
	for(i = 0, off = 0; i < newlength; i++, off += delta) \
		*newptr++ = ptr[off >> 16]; \
}

int resample(short *in, long int length, short *out,int speed)
  /* lenght is the number of SAMPLES, not bytes
     newlenght returned is number of BYTES */
{ 
	long int newlength;
	newlength = (length * 100) / speed;
	DO_PITCH(int, length, newlength, in, out);

	/* return newlength * 4 */
	return newlength<<2;
}
