/* A Gui in gtk+ for MuSE - Multiple Streaming Engine
 * Copyright (C) 2002 nightolo <night@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>

#include "generic.h"
#include "profile.h"
#include "gchan.h"
#include "gchan_utils.h"
#include "main.h"
#include "utils.h"
#include "musegtk.h"

#include <jutils.h>
#include <jmixer.h>
#include <config.h>

GtkWidget *window, *statusbar, *mic, *linein, *speakout;
guint contextid;
GtkWidget *maintab, *vbox;
GtkWidget *winil, *nbook, *mp3entry, *winprof;
unsigned int channum[7];/* from 1 to 6 now posnchan[pos]=channel idx */
unsigned int maxpos;
unsigned int numc;
Stream_mixer *mixer;
GList *listachan;
int guibitrate;
float storevol[MAX_CHANNELS];
char *pathfile;
struct lamesave *lametmp;
struct icesave *icetmp;
/* id of "select_row" handler */
guint blockid[7];
bool state;

bool gtkgui_init(int argc,char *argv[], Stream_mixer *mix)
{
	GtkWidget *bbox=NULL;
	bool isx=false;
	bool lastprof=false;
	char *lameprof=NULL;
	char *iceprof=NULL;
	char *p=NULL;
	int i;
	
	/* initialization */
	state=true;
	mixer=mix;
	for(i=0; i<7; i++) {
		channum[i]=0;
		blockid[i]=0;
	}
	maxpos=numc=0;
	guibitrate=24; 
	listachan=NULL;
	pathfile=NULL;

	/* signal to glib we're going to use threads */
	g_thread_init(NULL);

	isx=gtk_init_check(&argc,&argv);
	if(!isx) return false;	

	isx=mixer->set_lineout(true);

	/* profile init */
	profile_init();
	profile_init_lame(&lametmp);
	profile_init_ice(&icetmp);
	p=profile_get_default("lame");

	if(p) {
		lameprof=strdup(p);
		profile_get_lame(lameprof, lametmp);
		free(lameprof);
	} else {
		lastprof=profile_get_lame("last", lametmp);
		if(!lastprof) {
			notice("last lame profile not found, loading muse default");
			mixer->set_lame(24, 0, 3, 4, 0, 0);
			lametmp->status=true; /* profile_get_lame set status by itself */
		}
		else
			notice("last lame profile loaded");
	}
	p=NULL;
	p=profile_get_default("icecast");
	if(p) {
		iceprof=strdup(p);
		profile_get_ice(iceprof, icetmp);
		free(iceprof);
	}
	else {
		lastprof=profile_get_ice("last", icetmp);
		if(lastprof)
			notice("last icecast profile loaded");
	}

	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window),"MuSE - Multiple Streaming Engine");
	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
	gtk_container_set_border_width(GTK_CONTAINER(window),7);
	gtk_signal_connect(GTK_OBJECT(window),"delete_event",
					(GtkSignalFunc)gcb_exit,NULL);

	
	vbox=gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(window),vbox);
	
	bbox=createbbox(bbox);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, TRUE, TRUE, 0);
	if(isx)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(speakout), TRUE);
	
	createtab();	
	firstchanfree(channum);
	createch(1);
	putstatusbar();
	
	/*let's show window */
	gtk_widget_show_all(window);
	return true;	
}

bool
gtkgui_refresh(void)
{
  //  gdk_threads_enter();

  /* Rest in gtk_main and wait for fun to begin ;)  */
  while(gtk_events_pending())
    gtk_main_iteration();

  //  gdk_threads_leave();
  return state;
}

bool gtkgui_get_state(void)
{
	return state;
}

bool gtkgui_set_time(unsigned int chan, float position, char *testo)
{
	struct gchan *c;
	
	c=findch(chan+1, NOPOS);
	if(c) {
	  //	  gdk_threads_enter();
	  //	  func("gtkgui_set_time(%i,%f,%s)",chan,position,testo);
	  gtk_adjustment_set_value(GTK_ADJUSTMENT(c->adjprog), position );
	  gtk_entry_set_text(GTK_ENTRY(c->ptime), (gchar *) testo);

	  //	  gdk_threads_leave();
	  return true;
	}
	return false;
}

bool gtkgui_add_into_pl(unsigned int chan, char *file)
{
	struct gchan *o;

	char *cist[1];
	o=findch(chan+1, NOPOS);

	if(!o) return false;
	cist[0] = file;

	int riga = gtk_clist_append(GTK_CLIST(o->lista), cist);
	if(!riga) gtk_clist_select_row(GTK_CLIST(o->lista), 0, 0);

	return true;
}

void gtkgui_set_maintitle(char *testo) 
{
  //  gdk_threads_enter();

  gtk_window_set_title(GTK_WINDOW(window), (gchar *) testo);

  //  gdk_threads_leave();
}

void gtkgui_set_statustext(char *testo) 
{
  //  gdk_threads_enter();

  contextid=gtk_statusbar_push(GTK_STATUSBAR(statusbar), 
			       contextid, (gchar *) testo);
  
  //  gdk_threads_leave();
}

bool gtkgui_sel_playlist(unsigned int chan, int row)
{
	struct gchan *o;

	o=findch(chan+1, NOPOS);

	if(!o) return false;

	//	gdk_threads_enter();

	gtk_signal_handler_block(GTK_OBJECT(o->lista), blockid[chan+1]);
	gtk_clist_select_row(GTK_CLIST(o->lista), row-1, 0);
	gtk_signal_handler_unblock(GTK_OBJECT(o->lista), blockid[chan+1]);
	/* FIXME: use a function? */
	gtk_entry_set_text(GTK_ENTRY(o->ptime), "00:00");
	gtk_adjustment_set_value(GTK_ADJUSTMENT(o->adjprog), 0.0);
	
	//	gdk_threads_leave();
	return true;
}

void gtkgui_toggle_mic(bool i) {
  //  gdk_threads_enter();

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mic), i);

  //  gdk_threads_leave();
}

void gtkgui_toggle_spk(bool i) {
  //  gdk_threads_enter();

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linein), i);
  
  //  gdk_threads_leave();
}

void gtkgui_toggle_lineout(bool i) {
  //  gdk_threads_enter();

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(speakout), i);

  //  gdk_threads_leave();
}

void gtkgui_exit(void) {
  //  gdk_threads_enter();
  gtk_widget_destroy(window);
  g_list_free(listachan);
  profile_free_lame(lametmp);
  profile_free_ice(icetmp);
  
  //  gdk_threads_leave();

  
}
