/* A Gui in gtk+ for MuSE - Multiple Streaming Engine
 * Copyright (C) 2002 nightolo <night@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/limits.h>

#include "../jutils.h"
#include "profile.h"

/* my private functions, you mustn't see them */
FILE *profile_open(char *, bool tmp);
bool profile_close(FILE *, bool tmp);
char *profile_get_string(FILE *);

/* FIXME: more debugging */

/* the fun begins */
void profile_init(void)
{
	char *home;
	char file[PATH_MAX+NAME_MAX];
	FILE *fp;

	home=getenv("HOME");
	snprintf(file, sizeof(file), "%s/.muserc", home);
	
	fp=fopen(file, "r");

	if(!fp) {
		func("profile_init:: creating .muserc");
		fp=fopen(file, "w+");
		fclose(fp);
	}
}

FILE *profile_open(char *mode, bool tmp)
{
	char *home;
	char file[PATH_MAX+NAME_MAX];
	FILE *fp;
	
	home=getenv("HOME");
	if(!tmp)
		snprintf(file, sizeof(file), "%s/.muserc", home);
	else
		snprintf(file, sizeof(file), "%s/.muserc.tmp", home);
	
	if(!(fp=fopen(file, mode))) {
		error("profile_open:: fopen(%s, %s) returned NULL", file, mode);
		return NULL;
	}

	return fp;
}

bool profile_close(FILE *fp, bool tmp)
{
	char *home;	
	char file[PATH_MAX+NAME_MAX];
	
	if((fclose(fp))) {
		error("profile_close:: error closing file");
		return false;
	}
	
	if(tmp) {
		home=getenv("HOME");
		snprintf(file, sizeof(file), "%s/.muserc.tmp", home);
		if((unlink(file))) {
			error("profile_close:: error unlinking tmpfile");
			return false;
		}
	}

	return true;
}

char *profile_get_default(char *type)
{
	FILE *fp;
	char buf[250];
	char *s=NULL;
	char ptype[30];	

	fp=profile_open("r", false);

	if(!fp)
		return NULL;
	
	if( !(strcmp(type, "lame")) || !(strcmp(type, "icecast")))
		snprintf(ptype, sizeof(ptype), "%sdefault=", type);
	else {
		error("profile_get_default:: sorry profile type not supported yet : %s",
				type);
		return NULL;
	}
		
	while((fgets(buf, sizeof(buf), fp))) {
		if((strstr(buf, ptype))) {
			s = (char *) strchr(buf, '=')+1;
			s[strlen(s)-1] = '\0';
			func("profile_get_default(%s) returned %s", type, s);
			break;
		}
	}
			
	profile_close(fp, false);
	
	/* FIXME: remember to free */
	return s;
}

void profile_put_default(char *type, char *s)
{
	FILE *fp, *fptmp;
	char ptype[30], buf[250];
	
	if( !(strcmp(type, "lame")) || !(strcmp(type, "icecast"))) 
		snprintf(ptype, sizeof(ptype), "%sdefault=", type);
	else {
		error("profile_put_default:: sorry profile type not supported yet: %s",
				type);
		return;
	}
	
	if(!s)
		return;

	func("profile_put_default(%s, %s) called", type, s);

	fp=profile_open("r", false);
	fptmp=profile_open("w+", true);
	
	if(!(fp) || !(fptmp)) 
		return;
	
	while((fgets(buf, sizeof(buf), fp)))
		fprintf(fptmp, buf);

	profile_close(fp, false);

	fp=profile_open("w+", false);
	
	if(!fp)
		return;

	/* I've not to call profile_remove */
	(void)fseek(fptmp, 0L, SEEK_SET);
	
	fprintf(fp, "\n%s%s\n", ptype, s);
	
	while((fgets(buf, sizeof(buf), fptmp))) {
		if(!strstr(buf, ptype))
			fprintf(fp, buf);
	}

	profile_close(fp, false);
	profile_close(fptmp, true);
	
}

void profile_remove(char *s) 
{
	/* This function remove all lines from strstr(line, s) until
	 * line[0] != \n */
	FILE *fp, *fptmp;
	char buf[250];
	
	fp=profile_open("r", false);
	fptmp=profile_open("w+", true);

	if( !(fp) || !(fptmp) || !(s))
		return; 

	while((fgets(buf, sizeof(buf), fp)))
		fprintf(fptmp, buf);

	profile_close(fp, false);
	fp=profile_open("w+", false);

	fseek(fptmp, 0L, SEEK_SET);
	
	while((fgets(buf, sizeof(buf), fptmp))) {
		if(buf==NULL) {
			act("reached end of file, pattern not found");
			break;
		}
		if((strstr(buf, s))) {
			act("profile_remove:: pattern \"%s\" found!", s);
			while((fgets(buf, sizeof(buf), fptmp)) && *buf != '\n')
				;
		}
		if(buf)
			fprintf(fp, buf);
	}
	
	profile_close(fp, false);
	profile_close(fptmp, true);

	act("profile_remove:: I'm goin'!");
}

void profile_get_profile(char *type, char **pnam)
{
	FILE *fp;
	char *p;
	char ptype[10], buf[250];
	unsigned int i=0;

	for(i=0; i < 10; i++)
		pnam[i] = NULL;

	i=0;
	
	if( !(strcmp(type, "lame")) || !(strcmp(type, "icecast")))
		snprintf(ptype, sizeof(ptype), "[%s=", type);
	else {
		error("profile_get_profile:: sorry profile type not supported yet: %s",
				type);
		return; 
	}
	
	fp=profile_open("r+", false);
	
	if(fp) {
		while((fgets(buf, sizeof(buf), fp))) {
			if(i < 10 && (strstr(buf,ptype))) {
				p= (char *) strchr(buf, '=') + 1;
				p[strlen(p)-2]='\0';
/*				pnam[i]= (char *) malloc(strlen(p));*/
				pnam[i]=strdup(p);
				i++;
			}
		}
	}
	
	profile_close(fp, false);
	
}

void profile_free_pnam(char **pnam) 
{
	int i;	
	
	for(i=0; pnam[i] != NULL; i++)
		free(pnam[i]);
}

bool profile_init_lame(struct lamesave **lame)
{
	*lame= (struct lamesave *) malloc(sizeof(struct lamesave));
	if(!*lame) {
		error("profile_init_lame:: malloc return NULL");	
		return false;
	}
	(*lame)->bps=NULL;
	(*lame)->fr=NULL;
	(*lame)->mod=NULL;
	(*lame)->freqfil=NULL;
	(*lame)->profile=NULL;
	(*lame)->lps=NULL;
	(*lame)->hps=NULL;
	(*lame)->status=false;
	return true;
}
void profile_free_lame(struct lamesave *lame)
{
	if(lame->bps)
		free(lame->bps);
	if(lame->fr)
		free(lame->fr);
	if(lame->mod)
		free(lame->mod);
	if(lame->freqfil)
		free(lame->freqfil);
	if(lame->lps)
		free(lame->lps);
	if(lame->hps)
		free(lame->hps);
	if(lame->profile)
		free(lame->profile);
	if(lame)
		free(lame);
}

char *profile_get_string(FILE *fp)
{
	char *p;
	char buf[250];

	fgets(buf, sizeof(buf), fp);
	/* FIXME : newline doesn't work */
	if(buf[0] == '#' || buf[0] == '\n') {
		p=profile_get_string(fp);
		return p;
	}
	
	p= (char *) strchr(buf, '=') +1;
	p[strlen(p)-1]= '\0';
	
	return p;		

}

bool profile_get_lame(char *profile, struct lamesave *lame)
{
	FILE *fp;
	char buf[250];
	char *p;
	int c;

	if(!profile || !lame)
		return false;

	fp=profile_open("r+", false);

	if(!fp)
		return false;

	while((fgets(buf, sizeof(buf), fp))) {
		if((strstr(buf, "[lame="))) {
			p= (char *) strchr(buf, '=') + 1;
			p[strlen(p)-2]='\0';
			if((c=strcmp(p, profile)) == 0) {
				lame->bps=strdup(profile_get_string(fp));
				lame->fr=strdup(profile_get_string(fp));
				lame->mod=strdup(profile_get_string(fp));
				lame->freqfil=strdup(profile_get_string(fp));
				lame->lps=strdup(profile_get_string(fp));
				lame->hps=strdup(profile_get_string(fp));
				lame->profile=strdup(p);
				lame->quality=atoi(profile_get_string(fp)); /* FIXME */
				lame->status=true;
				profile_close(fp, false);
				return true;
			}
		}
	}
	func("profile_get_lame::sorry no profile found");
	profile_close(fp, false);
	return false;
}

bool profile_put_lame(char *profile, struct lamesave *lame)
{
	int c;
	FILE *fp;
	char proftype[30];

	if(!profile || !lame)
		return false;

	snprintf(proftype, sizeof(proftype), "[lame=%s]", profile);
	profile_remove(profile);
	
	fp=profile_open("r+", false);

	if(!fp)
		return false;

	if((c=fseek(fp, 0L, SEEK_END)) != 0) {
		func("profile_put_lame:: fseek problem");
		profile_close(fp, false);
		return false;
	}
	
	fprintf(fp, "\n[lame=%s]\n", profile);
	fprintf(fp, "bps=%s\nfr=%s\nmod=%s\nfreqfil=%s\nlps=%s\nhps=%s\nquality=%d\n\n", 
			lame->bps, lame->fr, lame->mod, lame->freqfil, 
			lame->lps, lame->hps, lame->quality);

	profile_close(fp, false);
	return true;
}

bool profile_init_ice(struct icesave **ice)
{
	*ice= (struct icesave *) malloc(sizeof(struct icesave));
	if(!*ice)
		return false;
	(*ice)->ip=NULL;
	(*ice)->port=NULL;
	(*ice)->mnt=NULL;
	(*ice)->pass=NULL;
	(*ice)->name=NULL;
	(*ice)->url=NULL;
	(*ice)->desc=NULL;
	(*ice)->login=NULL;
	(*ice)->profile=NULL;
	(*ice)->status=false;
	return true;
}

void profile_free_ice(struct icesave *ice)
{
	if(ice->ip)
		free(ice->ip);
	if(ice->port)
		free(ice->port);
	if(ice->mnt)
		free(ice->mnt);
	if(ice->pass)
		free(ice->pass); 
	if(ice->name)
		free(ice->name);
	if(ice->url)
		free(ice->url);
	if(ice->desc)
		free(ice->desc);
	if(ice->login)
		free(ice->login);
	if(ice->profile)
		free(ice->profile);
	if(ice)
		free(ice);

}

bool profile_get_ice(char *profile, struct icesave *ice)
{
	FILE *fp;
	char buf[250];
	char *p;
	int c;

	if(!profile || !ice)
		return false;

	fp=profile_open("r+", false);

	if(!fp)
		return false;

	while((fgets(buf, sizeof(buf), fp))) {
		if((strstr(buf, "[icecast="))) {
			p= (char *) strchr(buf, '=') + 1;
			p[strlen(p)-2]='\0';
			if((c=strcmp(p, profile)) == 0) {
				func("profile_get_ice::profile %s match in %s!", p, profile);
				ice->ip=strdup(profile_get_string(fp));
				ice->port=strdup(profile_get_string(fp));
				ice->mnt=strdup(profile_get_string(fp));
				/* LOOKAT: I don't get password */
				ice->name=strdup(profile_get_string(fp));
				ice->url=strdup(profile_get_string(fp));
				ice->desc=strdup(profile_get_string(fp));
				ice->login=strdup(profile_get_string(fp));
				ice->logtype=atoi(profile_get_string(fp));
				ice->profile=strdup(p);
				ice->status=true;
				profile_close(fp, false);
				return true;
			}
		}
	}

	func("profile_get_ice:: sorry no profile found", ice);
	profile_close(fp, false);
	return false;
}

bool profile_put_ice(char *profile, struct icesave *ice)
{
	int c;
	FILE *fp;
	char proftype[30];

	if(!profile || !ice)
		return false;

	snprintf(proftype, sizeof(proftype), "[icecast=%s]", profile);
	profile_remove(profile);
	
	fp=profile_open("r+", false);

	if(!fp)
		return false;
	
	if((c=fseek(fp, 0L, SEEK_END)) != 0) {
		func("profile_put_ice:: fseek problem");
		profile_close(fp, false);
		return false;
	}

	fprintf(fp, "\n[icecast=%s]\n", profile);
	fprintf(fp, "ip=%s\nport=%s\nmnt=%s\nname=%s\ndesc=%s\nurl=%s\nlogin=%s\nlogtype=%d\n\n", 
			ice->ip, ice->port, ice->mnt, ice->name, ice->desc, ice->url,
			ice->login, ice->logtype);

	profile_close(fp, false);
	return true;
}
	
