/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __JMIXER_H
#define __JMIXER_H

#include <pthread.h>

#include <inchannels.h>
#include <outchannels.h>
#include <gui.h>
#include <playlist.h>

#include <generic.h>

class Stream_mixer {

 private:
  
  GUI *gui;
  bool have_gui;

  int dsp;
  int max; /* lenght of actual buffer in bytes */
  int peak; /* actual volume peak */
  bool fullduplex;

  LiveIn *livein;
  
  void close_soundcard();
  void updchan(int ch);
  void clip_audio(int chunks);
  int get_level(int chunks);
  
  pthread_mutex_t _mutex;
  pthread_cond_t _cond;

 public:
  Stream_mixer();
  ~Stream_mixer();

  /* pthread stuff */
  void lock() { pthread_mutex_lock(&_mutex); };
  void unlock() { pthread_mutex_unlock(&_mutex); };
  void wait() { pthread_cond_wait(&_cond,&_mutex); };
  void signal() { pthread_cond_signal(&_cond); };
  /* ------------- */

  void register_gui(GUI *reg_gui);
  bool open_soundcard();
  bool set_icecast(char *ip, int port, char *pass, char *mountpoint, int bps, char *name, char *url, char *desc, int login);
  void set_lame(int bps, int freq, int mode, int quality, int lowpass, int highpass);
  void stop_icecast();
  int set_mp3file(char *file, int bps);
  void stop_mp3file();
  void cafudda();
  int set_channel(int ch, int playlist_pos);
  bool create_channel(int ch);
  void delete_channel(int ch);
  bool play_channel(int ch);
  bool stop_channel(int ch);
  void pause_channel(int ch);
  void pause_channel(int ch, bool stat);
  void set_all_volumes(float *vol);
  void set_volume(int ch, float vol);
  void set_speed(int ch, int speed);
  bool set_position(int ch, float pos);
  bool set_live(bool stat);
  bool set_lineout(bool stat);
  bool set_playlist(char *file, int ch);
  void set_playmode(int ch, int mode);
  bool add_to_playlist(int ch, char *file);
  void rem_from_playlist(int ch, int pos);

  /* notice if we have a soundcard */
  bool dsp_ok() { if(dsp>0) return true; else return false; };

  /* channels and playlists */
  Channel *xach[MAX_CHANNELS];
  Playlist playlist[MAX_CHANNELS];

  /* encoder outchannel */
  LameEnc lame;

  /* ice shouter */
  IcecastOut ice;

  /* file dump */
  Mp3fileOut mp3file;

  bool dspout;
  bool linein;
  bool icecastout;
  bool fileout;
  bool quit;
  
  /* that's the socket registered from netlistener */
  int *client;

  /* mixing buffers */
  short audio_buffer[PROCBUF_SIZE];
  int process_buffer[PROCBUF_SIZE];

};

#endif
