/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sched.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <jutils.h>
#include <gui.h>

#include <config.h>

#define MAX_DEBUG 2

#define FUNC 2 /* se il debug level e' questo
		  ci sono le funzioni chiamate */
#define WARN 1 /* ... blkbblbl */

static int verbosity;
static GUI *gui = NULL;

void set_debug(int lev) {
  lev = lev<0 ? 0 : lev;
  lev = lev>MAX_DEBUG ? MAX_DEBUG : lev;
  verbosity = lev;
}

int get_debug() {
  return(verbosity);
}

void set_guimsg(GUI *g) {
  gui = g;
}

void notice(char *format, ...) {
  char msg[255];
  va_list arg;
  va_start(arg, format);

  vsnprintf(msg, 254, format, arg);
  cerr << "[*] " << msg << endl;

  va_end(arg);
  if(gui) gui->set_status(msg);
}

void func(char *format, ...) {
  if(verbosity>=FUNC) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsnprintf(msg, 254, format, arg);
    cerr << "[F] " << msg << endl;
    
    va_end(arg);
  }
}

void error(char *format, ...) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsnprintf(msg, 254, format, arg);
    cerr << "[!] " << msg << endl;
    //    perror(msg);    

    va_end(arg);
}

void act(char *format, ...) {
  char msg[255];
  va_list arg;
  va_start(arg, format);
  
  vsprintf(msg, format, arg);
  cerr << " .  " << msg << endl;
  
  va_end(arg);
}

void warning(char *format, ...) {
  if(verbosity>=WARN) {
    char msg[255];
    va_list arg;
    va_start(arg, format);
    
    vsprintf(msg, format, arg);
    cerr << "[W] " << msg << endl;
  
    va_end(arg);
  }
}

double dtime() {
  struct timeval mytv;
  gettimeofday(&mytv,NULL);
  return((double)mytv.tv_sec+1.0e-6*(double)mytv.tv_usec);
}

void jsleep(int sec, long nsec) {
  timespec timelap;
  timelap.tv_sec = sec;
  timelap.tv_nsec = nsec;
  nanosleep(&timelap,NULL);
}

/* sets the process to "policy" policy,  if max=1 then set at max priority,
   else use min priority */

bool set_rtpriority(bool max) {
  struct sched_param schp;
  /*
   * set the process to realtime privs
   */
  memset(&schp, 0, sizeof(schp));
  
  if(max) 
    schp.sched_priority = sched_get_priority_max(SCHED_RR);
  else
    schp.sched_priority = sched_get_priority_min(SCHED_RR);
  
  if (sched_setscheduler(0, SCHED_RR, &schp) != 0)
    return false;
  else
    return true;
}
