/* MuSE - Multiple Streaming Engine
 * Copyright (C) 2000-2002 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * here command line is parsed
 * synchronous threads are launched depending on mode selection
 * signals are trapped for nice&clean exiting
*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <signal.h>

#include <jutils.h>
#include <generic.h>
#include <jmixer.h>
#include <gui.h>

#ifdef GUI_NIGHTOLO
#include <gtkgui/gtk_gui.h>
#define GOT_GUI 1
#endif

#include <config.h>

/* command line stuff */

char *version =
"%s version %s [ http://muse.dyne.org ]";

char *help =
"Usage: muse [generic options] [-c [encoder options] [stream options] [files]]\n"
":: generic options:\n"
" -h --help         this help\n"
" -v --version      version information\n"
" -d --debug [1-3]  debug verbosity level       - default 3\n"
" -o --dspout       disable souncard output\n"
" -c --cli          command line input (no GUI)\n"
" -g --gui          specify GUI to use (-g list)\n"
":: encoder options:\n"
" -b --bitrate      encoding bitrate in Kbit/s  - default 24\n"
" -q --quality      encoding quality (1-9)      - default 4\n"
" -f --filedump     dump stream to file\n"
" -i --live         run listening on live input\n"
":: stream options:\n"
" -s --server       stream to server\n"
" -p --port         server port                 - default 8000\n"
" -m --mountpoint   mounpoint on server         - default live\n"
" -x --pass         encoder password on server\n"
" -n --name         name of the stream\n"
"\n";

static const struct option long_options[] = {
  { "help", no_argument, NULL, 'h' },
  { "version", no_argument, NULL, 'v' },
  { "debug", required_argument, NULL, 'd' },
  { "dspout", no_argument, NULL, 'o' },
  { "bitrate", required_argument, NULL, 'b' },
  { "quality", required_argument, NULL, 'q' },
  { "filedump", required_argument, NULL, 'f' },
  { "cli", no_argument, NULL, 'c' },
  { "gui", required_argument, NULL, 'g' },
  { "live", no_argument, NULL, 'i' },
  { "server", required_argument, NULL, 's' },
  { "port", required_argument, NULL, 'p' },
  { "mountpoint", required_argument, NULL, 'm' },
  { "pass",required_argument, NULL, 'x' },
  { "name",required_argument, NULL, 'n' },
  { 0, 0, 0, 0 }
};

char *short_options = "-hvd:ob:q:f:cg:is:p:m:x:n:";

/* misc settings */
#define MAX_CLI_CHARS 9182
int debug = 3;
bool daemon_mode = false;
char *queue_file = NULL;

int brate = 24;
int quality = 2;
char *filedump = NULL;

int thegui = -1;

bool live = false;
bool dspout = true;
bool snddev = false;
char files[MAX_CLI_CHARS];
int cli_chars = 0;
char *server = NULL;
int port = 8000;
char *mountpoint = NULL;
char *pass = NULL;
char *name = NULL;


Stream_mixer *mix;
GUI *gui;

/* declare the sighandler */
void quitproc (int Sig);
volatile sig_atomic_t userbreak;
/* ---------------------- */

void take_args(int argc, char **argv) {
  notice(version,PACKAGE,VERSION);
  act("by Denis Rojo aka jaromil <jaromil@dyne.org>");

  int res;
  memset(files,0,MAX_CLI_CHARS);
  char *fp = files;

  set_debug(3);
  
  do {
    res = getopt_long(argc, argv, short_options, long_options, NULL);

    switch(res) {
    case 'h':
      fprintf(stderr,"%s",help);
      exit(0);
      break;
    case 'v':
      exit(0);
      break;
    case 'd':
      debug = atoi(optarg);
      break;
    case 'o':
      dspout = false;
      break;
    case 'b':
      brate = atoi(optarg);
      break;
    case 'q':
      quality = atoi(optarg);
      if(quality<1) quality = 1;
      if(quality>9) quality = 9;
      break;
    case 'f':
      filedump = strdup(optarg);
      break;
    case 'c':
      thegui = 0;
      break;
    case 'g':
      if(strcmp(optarg,"list")==0) {
	notice("listing available user interfaces:");
#ifdef GUI_NIGHTOLO
	act("[gtk] - Xwin graphical interactive clicky clicky");
#endif
	act("[cli] - command line interface, not interactive");
	exit(0);
      }
      if(strcmp(optarg,"cli")==0) thegui=0;
#ifdef GUI_NIGHTOLO
      if(strcmp(optarg,"gtk")==0) thegui=1;
#endif
      break;
    case 'i':
      live = true;
      break;
    case 's':
      server = strdup(optarg);
      break;
    case 'p':
      port = atoi(optarg);
      break;
    case 'm':
      mountpoint = strdup(optarg);
      break;
    case 'x':
      pass = strdup(optarg);
      break;
    case 'n':
      name = strdup(optarg);
      break;
    case 1:
      {
	int optlen = strlen(optarg);
	if( (cli_chars+optlen) < MAX_CLI_CHARS ) {
	  sprintf(fp,"%s#",optarg);
	  cli_chars+=optlen+1;
	  fp+=optlen+1;
	} else {
	  warning("too much files on commandline to insert in a playlist");
	  warning("list truncated, consider including them into a .pls file");
	}
      }
      break;
    default:
      break;
    }
  } while(res > 0);  
}

int main(int argc, char **argv) {
  bool run_ok = false;
  mix = NULL;
  gui = NULL;

  take_args(argc, argv);

  /* register signal traps */
  if (signal (SIGINT, quitproc) == SIG_ERR) {
    error ("Couldn't install SIGINT handler"); exit (0);
  }

  if (signal (SIGQUIT, quitproc) == SIG_ERR) {
    error ("Couldn't install SIGQUIT handler"); exit (0);
  }

  if (signal (SIGTERM, quitproc) == SIG_ERR) {
    error ("Couldn't install SIGTERM handler"); exit (0);
  }

  set_debug(debug);

  act("starting engine...");
  mix = new Stream_mixer();
  
  snddev = mix->open_soundcard();
  if(!snddev) {
    warning("no soundcard found");
    act("line-in and speaker out deactivated");
  }
  
#ifdef GOT_GUI
  if((thegui<0) || (thegui==1)) 
    if(getenv("DISPLAY")) {
      thegui=1;
      notice("spawning the GTK+ graphical user interface");
      act("by nightolo <night@autistici.org>");	  
    } else { error("no X display found"); thegui = 0; }
  
  if(thegui>0) {
    switch(thegui) {
#ifdef GUI_NIGHTOLO
    case 1:
      gui = new GTK_GUI(argc,argv,mix);
      break;
#endif
    }
    gui->start();
    mix->register_gui(gui);
    set_guimsg(gui);
    notice("%s version %s",PACKAGE,VERSION);
    if(cli_chars>0) {
      char *p, *pp = files;
      while(cli_chars>0) {
	p = pp; while(*p!='#' && cli_chars>0) { p++; cli_chars--; }
	if(cli_chars<=0) break; *p='\0';
	mix->add_to_playlist(0,pp);
	pp = p+1;
      }
    }
    run_ok = true;
  } else { /* CLI interface logics ======================= */
#endif
    /* lame guesses for best frequency
       hardcoded mono channel (mode 3)
       lame guesses for best low/highpass filtering
    */
    if(!cli_chars && !live) {
      error("CLI: nothing to stream! no files and no live input have been selected");
      fprintf(stderr,"%s",help);
      exit(0);
    }
    
    if(snddev) {
      mix->set_lineout(false); /* linein has priority */
      if( mix->set_live(live) ) notice("CLI: recording from mic/linein");
      if( mix->set_lineout(dspout) ) notice("CLI: playing on soundcard");
    }

    if(server||filedump) mix->set_lame(brate,0,3,quality,0,0);
    if(filedump) mix->set_mp3file(filedump,brate);

    if(cli_chars>0) { /* there are some files to be played */
      if(!mix->create_channel(0))
	error("CLI: strange! i got problems creating a channel");
      mix->set_playmode(0,2); /* CONTINUOUS playing mode, cycle thru playlist */
      
      char *pp = files, *p;
      while(cli_chars>0) {
	p = pp; while(*p!='#' && cli_chars>0) { p++; cli_chars--; }
	if(cli_chars<=0) break; *p='\0';
	if(mix->add_to_playlist(0,pp))
	  func("CLI: added %s into playlist",pp);
	pp = p+1;
      }
      
      if(mix->set_channel(0,1)==0)
	error("CLI: fatal error setting channel!");
      else {
	jsleep(0,100); mix->play_channel(0); 
	run_ok = true;
      }
    } else run_ok = true;
    
#ifdef GOT_GUI
  }
#endif

  if(server && run_ok)
    mix->set_icecast(server,port,pass,mountpoint,brate,name,NULL,NULL,0);
  
  /* MAIN LOOP */
  if(run_ok)
    while(!mix->quit)
      mix->cafudda();
  /* simple isn't it? */
  notice("quitting MuSE");
  set_guimsg(NULL);
  if(gui) gui->quit = true;

  jsleep(2,0);

  if(mix) delete mix;
  if(gui) delete gui;
  
  if(filedump) free(filedump);
  if(server) free(server);
  if(mountpoint) free(mountpoint);
  if(pass) free(pass);
  if(name) free(name);
  
  act("cya!");
  
  exit(0);
}

/* signal handling */
void quitproc (int Sig) {
  func("received signal %u on process %u",Sig,getpid());
  if(thegui>0) gui->quit = true;
  mix->quit = true;
}
