/*
  Copyright (c) 2001 Charles Samuels <charles@kde.org>
  Copyright (c) 2002 Denis Rojo <jaromil@dyne.org>
  
this pipe class was first written by Charles Samuels
and then heavily mutilated and optimized by Denis Rojo

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
  
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
   
You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#ifndef __PIPE_H__
#define __PIPE_H__

#include <pthread.h>
#include <string.h>

class Pipe {
public:
  Pipe(int size=16384);
  ~Pipe();
 

  int read(int length, void *data);
  
  /**
   * calls the above read with force=false, then
   * fills all remaining space with the given byte
   **/
  int read(int length, char fill, void *data);
  
  //	int peek(int length, void *data, bool block=true) const; // TODO
  int write(int length, void *data);
  
  
  int size() const;
  int space() const;
  
  void flush();
  void flush(int size);
    
 private:
  pthread_mutex_t mMutexStart, mMutexEnd, mMutexRead, mMutexWrite;
  pthread_cond_t mCondStart, mCondEnd;
  
  void *buffer;
  void *bufferEnd;
  
  void *start;
  void *end;
};

#endif
